package uicc.contactmanager;

/**
 * <p>
 *   A contact list is the collection of all contacts managed by a contact manager.
 * </p>
 * <p>
 *   Within a contact list, each contact is assigned a numeric identifier,
 *   that is unique within the contact list.
 *   <br>
 *   The identifier for each contact is assigned by the containing contact list
 *   and is only significant within the parent contact manager.
 * </p>
 * <p>
 *   An object that realizes the <code>ContactsListView</code> interface represents a contact list.
 *   <br>
 *   It represents the same contacts list since its creation, during its whole lifetime.
 * </p>
 * <p>
 *   Each <code>ContactsListView</code> instance provides an independent enumeration scheme 
 *   to enumerate all contacts that belong to the list, or a subset of them defined by 
 *   specified filtering criteria.
 * </p>
 * <p>
 *   <code>ContactsListView</code> instances also provide a factory method to create an instance 
 *   of the <code>ContactView</code> interface.
 * </p>
 * <p>
 *   Multiple <code>ContactsListView</code> instances may be created by the same or different 
 *   applications. Although they may represent the same contacts list, each of them has its 
 *   own state.
 *   <br>
 *   This particularly applies to the current enumeration progress, which is specific to
 *   each instance.
 * </p>
 */
public interface ContactsListView {
    /**
     * <p>
     *   Factory method used to create an instance of the <code>ContactView</code> interface.
     * </p>
     * <p>
     *   The instance is initially created in the DESELECTED state.
     *   <br> 
     *   See the description of the <code>ContactView</code> interface for detailed information
     *   about its complete life cycle.
     * </p>
     * <p>
     *   At its creation, the <code>ContactView</code> instance, is readily setup with the list of 
     *   fields specified in the field descriptors list associated to the contact manager related to
     *   the contacts list instance that created it.
     * </p>
     * <p>
     *   The new instance is owned by the calling application.
     * </p>
     *
     * @return new <code>ContactView</code> instance
     */
    public ContactView createContactViewInstance();

    /**
     * <p>
     *   Starts a new enumeration of all contacts in the list.
     * </p>
     * <ul>
     *   <li>A <code>ContactsListView</code> instance can process one enumeration at a time.<br>
     *       If an enumeration is already in progress, it is reset to the first contact
     *       in the list.</li>
     *   <li>The next element in the enumeration can be retrieved with the method 
     *       <code>selectNextContact()</code>.</li>
     * </ul>
     */
    public void startEnumeration();

    /**
     * <p>
     *   Starts a new enumeration of contacts that match specific criteria.
     * </p>
     * <p>
     *   Only contacts, for which a field matches a specific type, attributes and a value pattern,
     *   will participate in the enumeration.
     *   <br>
     *   The type, attributes and value pattern matching rules are described in the <code>ContactView</code> 
     *   interface's description.
     *   <br>
     *   A contact participates in the enumeration if at least one of its fields matches the 
     *   type, attributes and value pattern specified.
     * </p>
     * <p>
     *   In addition to the filtering mechanism described here, all requirements of the method 
     *   <code>startEnumeration()</code> apply to this method.
     * </p>
     * 
     * @param type the type of the field searched for
     * @param attributesMask mask of attributes of fields searched for
     * @param pattern pattern of field values to look for; if null, all field values are accepted
     * @param patternOffset pattern Offset 
     * @param patternLength pattern length
     */
    public void startEnumeration(short type, int attributesMask, byte [] pattern, short patternOffset, short patternLength);

    /**
     * <p>
     *   Checks if there are more contacts in the current contact enumeration.
     * </p>
     * 
     * @return <code>true</code> if there is at least one more contact in the current enumeration
     */
    public boolean hasMoreContacts();

    /**
     * <p>
     *   Binds a <code>ContactView</code> instance with the next contact in the current enumeration.
     * </p>
     * <p>
     *   The order in which the contacts are enumerated is unspecified.
     * </p>
     * <p>
     *   After successful operation, the <code>ContactView</code> instance is in the SELECTED state, 
     *   and is bound to the next contact in the current enumeration.
     * </p>
     * <p>
     *   If the current enumeration has not been started, or already reached its end,
     *   or after unsuccessful operation for any reason, the <code>ContactView</code> instance
     *   is in DESELECTED state.
     * </p>
     * 
     * @param contactView <code>ContactView</code> instance that should represent the next contact in the enumeration.
     * 
     * @return contact identifier, or -1 if the current enumeration reached its end.
     */
    public int selectNextContact(ContactView contactView);

    /**
     * <p>
     *   Binds a <code>ContactView</code> instance to the contact that matches the provided 
     *   <code>contactIdentifier</code>.
     * </p>
     * <p>
     *   After successful operation, the <code>ContactView</code> instance is in SELECTED state, 
     *   and is the contact specified is selected.
     * </p>
     * <p>
     *   After unsuccessful operation, the <code>ContactView</code> instance is in DESELECTED state.
     * </p>
     * 
     * @param contactView <code>ContactView</code> instance that should represent the next contact that
     *                matches the identifier. 
     * @param contactIdentifier contact identifier of contact to get
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>:
     *    <ul>
     *      <li>with reason <code>WRONG_IDENTIFIER</code> if the list contains no contact with the 
     *          <code>contactIdentifier</code> provided.</li>
     *    </ul>
     */
    public void selectContact(ContactView contactView, int contactIdentifier);

    /**
     * <p>
     *   Removes a contact from the contacts list.
     * </p>
     * <p>
     *   If the removed contact also belongs to one or more groups,
     *   it is also removed from these groups.
     * </p>
     * <p>
     *   This method fails in the following cases:
     *   <ul>
     *     <li>
     *       If any <code>ContactView</code> instance is currently bound to the removed contact.</li>
     *     <li>
     *       If an enumeration is currently in progress in any instance of <code>ContactsListView</code>
     *       or <code>GroupView</code> related the same contact manager, other than the method's
     *       target.
     *     </li>
     *   </ul>
     * </p>
     * 
     * @param contactIdentifier identifier of the contact to be removed from the list
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code> with the following reasons:
     *   <ul>
     *      <li><code>WRONG_IDENTIFIER</code> if <code>contactIdentifier</code>
     *          does not identify any contact in the contacts list.</li>
     *      <li><code>ITEM_SELECTED</code> if the contact to be removed is currently selected 
     *        by a <code>ContactView</code>.</li>
     *      <li><code>ENUMERATION_IN_PROGRESS</code> if a conflicting enumeration is
     *          currently in progress.</li>
     *   </ul>
     */
    public void removeContact(int contactIdentifier);

    /**
     * <p>
     *   Adds a contact to the contacts list.
     * </p>
     * <p>
     *   The contact to be added is represented by an instance of <code>ContactView</code>
     *   that is in DESELECTED state, whose fields have been set to the appropriate values.
     * </p>
     * <p>
     *   When a contact is added to a contacts list, any enumeration over the same contacts 
     *   list currently in progress may or may not include the new contact, depending on the 
     *   implementation.
     * </p>
     * <p>
     *   After successful completion of this operation, the <code>ContactView</code> instance is 
     *   in SELECTED state, and is bound to the newly created contact.
     * </p>
     *
     * @param contactView A <code>ContactView</code> instance with <code>FieldView</code> values set according 
     *                to the contact to be added.
     * 
     * @return the new contact's identifier
     */ 
    public int addContact(ContactView contactView);

    /**
     * <p>
     *   Returns the number of contacts in the contacts list.
     * </p>
     * 
     * @return number of contacts
     */
    public int getNumberOfContacts();
}
