package uicc.contactmanager;


/**
 * <p>
 *   Groups are collections of contacts from a contact list.
 * </p>
 * <p>
 *   The groups are very similar to the contact list; however, while the contact list
 *   contains all the contact manager's contacts, groups only represent a subset of 
 *   the contacts contained in the contact manager's contact list.
 *   <br>
 *   Therefore, while a contact always belongs to a unique contact list, it may belong 
 *   to one or more groups, or none.
 *   <br>
 *   While contacts are assigned a unique identifier by their contact list,
 *   the very same unique identifier is used to identify them in all the groups 
 *   they may belong to.
 * </p>
 * <p>
 *   Each group belongs to a unique group list.
 * </p>
 * <p>
 *   In addition to the methods inherited from <code>ContactsListView</code>, the 
 *   <code>GroupView</code> class provides methods to retrieve and set a group name.
 * </p>
 * <p>
 *   Because of the different ownership rules described above between the contacts list
 *   and the groups, the semantics of methods inherited from <code>ContactsListView</code> slightly differ. 
 *   These differences are listed below:
 *   <ul>
 *     <li>The <code>addContact()</code> method can only operate on <code>ContactView</code> instances 
 *         in the SELECTED state, that represent contacts already in the contact list. A reference to 
 *         the contact is added to the group.</li>
 *     <li>The <code>addContact()</code> method removes a contact from the group. The contact remains 
 *         in its contact list.</li>
 *   </ul>
 * </p>
 * <p>
 *   A <code>GroupView</code> instance may successively represent multiple groups
 *   throughout its lifetime.
 *   <br>
 *   It is good practice, for an application that needs to manipulate 
 *   groups, to create one <code>GroupView</code> instance at installation, that will
 *   be re-used throughout the application's lifetime.
 * </p>
 * <p>
 *   An instance is necessarily in one of the following states:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing group in the groups list.
 *         This is the initial state after creation of a new instance.</li>
 *     <li>SELECTED: the instance represents a group in the groups list.</li>
 *   </ul>
 * </p>
 * <p>
 *   More detailed description of the behavior of a <code>GroupView</code> instance in these states is 
 *   given below:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing group in the groups list.
 *         This is the initial state after creation of a new instance.
 *       <ul>
 *         <li>The identifier is -1.</li>
 *         <li>No operation is permitted on the instance.</li>
 *       </ul>
 *     </li>
 *     <li>SELECTED: the instance represents a group in the groups list.
 *       <ul>
 *         <li>The instance enters this state after successful execution of one of these methods:
 *           <ul>
 *             <li><code>GroupsListView.getGroup()</code></li>
 *             <li><code>GroupsListView.nextGroup()</code></li>
 *           </ul>
 *         </li>
 *         <li>All operations are allowed.</li>
 *         <li>The instance goes back to the DESELECTED state at the following events:
 *           <ul>
 *             <li><code>release()</code></li>
 *             <li>A RESET event occurred.</li>
 *           </ul>
 *         </li> 
 *       </ul>
 *     </li>
 *   </ul>
 * </p>
 */
public interface GroupView extends ContactsListView {
    /**
     * Sets the group name.
     * @param name name of the group to set
     * @param nameOffset offset in name buffer
     * @param nameLength length of name buffer
     */
    public void setGroupName(byte [] name, short nameOffset, short nameLength);

    /**
     * Gets the group name.
     * 
     * @param name buffer to store the name into
     * @param nameOffset offset in name buffer
     * @param maxNameLength maximum length of name buffer
     * 
     * @throws ArrayIndexOutOfBoundException if nameOffset+the length of the group name > maxNameLength
     */
    public void getGroupName(byte [] name, short nameOffset, short maxNameLength);

    /**
     * <p>
     *   If this instance is in SELECTED state, the selected group is deselected.
     * </p>
     * <p>
     *   After a successful call to this method, the instance is in DESELECTED state.
     * </p>
     */
    public void deselect();
}
